@@ -46,6 +46,8 @@
 #include "tracker/tracker_manager.h"
 #include "torrent/exceptions.h"
 #include "torrent/data/file_list.h"
+#include "torrent/peer/connection_list.h"
+#include "torrent/peer/peer.h"
 #include "torrent/peer/peer_info.h"
 
 #include "available_list.h"
@@ -104,6 +106,12 @@ DownloadMain::~DownloadMain() {
   if (m_taskTrackerRequest.is_queued())
     throw internal_error("DownloadMain::~DownloadMain(): m_taskTrackerRequest is queued.");
 
+  // Check if needed.
+  m_connectionList->clear();
+
+  if (m_info->size_pex() != 0)
+    throw internal_error("DownloadMain::~DownloadMain(): m_info->size_pex() != 0.");
+
   delete m_trackerManager;
   delete m_uploadChokeManager;
   delete m_downloadChokeManager;
@@ -116,17 +124,14 @@ DownloadMain::~DownloadMain() {
 
   m_ut_pex_delta.clear();
   m_ut_pex_initial.clear();
-
-  if (m_info->size_pex() != 0)
-    throw internal_error("DownloadMain::~DownloadMain(): m_info->size_pex() != 0.");
 }
 
 void
-DownloadMain::open() {
+DownloadMain::open(int flags) {
   if (info()->is_open())
     throw internal_error("Tried to open a download that is already open");
 
-  file_list()->open();
+  file_list()->open(flags & FileList::open_no_create);
 
   m_chunkList->resize(file_list()->size_chunks());
   m_chunkStatistics->initialize(file_list()->size_chunks());
@@ -237,11 +242,11 @@ DownloadMain::receive_connect_peers() {
 
   while (!peer_list()->available_list()->empty() &&
          manager->connection_manager()->can_connect() &&
-         connection_list()->size() < connection_list()->get_min_size() &&
-         connection_list()->size() + m_slotCountHandshakes(this) < connection_list()->get_max_size()) {
+         connection_list()->size() < connection_list()->min_size() &&
+         connection_list()->size() + m_slotCountHandshakes(this) < connection_list()->max_size()) {
     rak::socket_address sa = peer_list()->available_list()->pop_random();
 
-    if (connection_list()->find(sa) == connection_list()->end())
+    if (connection_list()->find(sa.c_sockaddr()) == connection_list()->end())
       m_slotStartHandshake(sa, this);
   }
 }
@@ -257,7 +262,7 @@ DownloadMain::receive_tracker_success() {
 
 void
 DownloadMain::receive_tracker_request() {
-  if (connection_list()->size() >= connection_list()->get_min_size())
+  if (connection_list()->size() >= connection_list()->min_size())
     return;
 
   if (m_info->is_pex_enabled() || connection_list()->size() < m_lastConnectedSize + 10)
@@ -284,7 +289,7 @@ DownloadMain::do_peer_exchange() {
   int togglePex = 0;
 
   if (!m_info->is_pex_active() &&
-      m_connectionList->size() < m_connectionList->get_min_size() / 2 &&
+      m_connectionList->size() < m_connectionList->min_size() / 2 &&
       m_peerList.available_list()->size() < m_peerList.available_list()->max_size() / 4) {
     m_info->set_pex_active(true);
 
@@ -293,7 +298,7 @@ DownloadMain::do_peer_exchange() {
       togglePex = PeerConnectionBase::PEX_ENABLE;
 
   } else if (m_info->is_pex_active() &&
-             m_connectionList->size() >= m_connectionList->get_min_size()) {
+             m_connectionList->size() >= m_connectionList->min_size()) {
 //              m_peerList.available_list()->size() >= m_peerList.available_list()->max_size() / 2) {
     togglePex = PeerConnectionBase::PEX_DISABLE;
     m_info->set_pex_active(false);
@@ -304,31 +309,32 @@ DownloadMain::do_peer_exchange() {
   ProtocolExtension::PEXList current;
 
   for (ConnectionList::iterator itr = m_connectionList->begin(); itr != m_connectionList->end(); ++itr) {
-    const rak::socket_address* sa = rak::socket_address::cast_from((*itr)->peer_info()->socket_address());
+    PeerConnectionBase* pcb = (*itr)->m_ptr();
+    const rak::socket_address* sa = rak::socket_address::cast_from(pcb->peer_info()->socket_address());
 
-    if ((*itr)->peer_info()->listen_port() != 0 && sa->family() == rak::socket_address::af_inet)
-      current.push_back(SocketAddressCompact(sa->sa_inet()->address_n(), (*itr)->peer_info()->listen_port()));
+    if (pcb->peer_info()->listen_port() != 0 && sa->family() == rak::socket_address::af_inet)
+      current.push_back(SocketAddressCompact(sa->sa_inet()->address_n(), pcb->peer_info()->listen_port()));
 
-    if (!(*itr)->extensions()->is_remote_supported(ProtocolExtension::UT_PEX))
+    if (!pcb->extensions()->is_remote_supported(ProtocolExtension::UT_PEX))
       continue;
 
     if (togglePex == PeerConnectionBase::PEX_ENABLE) {
-      (*itr)->set_peer_exchange(true);
+      pcb->set_peer_exchange(true);
 
       if (m_info->size_pex() >= m_info->max_size_pex())
         togglePex = 0;
 
-    } else if (!(*itr)->extensions()->is_local_enabled(ProtocolExtension::UT_PEX)) {
+    } else if (!pcb->extensions()->is_local_enabled(ProtocolExtension::UT_PEX)) {
       continue;
 
     } else if (togglePex == PeerConnectionBase::PEX_DISABLE) {
-      (*itr)->set_peer_exchange(false);
+      pcb->set_peer_exchange(false);
 
       continue;
     }
 
     // Still using the old buffer? Make a copy in this rare case.
-    DataBuffer* message = (*itr)->extension_message();
+    DataBuffer* message = pcb->extension_message();
 
     if (!message->empty() && (message->data() == m_ut_pex_initial.data() || message->data() == m_ut_pex_delta.data())) {
       char* buffer = new char[message->length()];
@@ -336,7 +342,7 @@ DownloadMain::do_peer_exchange() {
       message->set(buffer, buffer + message->length(), true);
     }
 
-    (*itr)->do_peer_exchange();
+    pcb->do_peer_exchange();
   }
 
   std::sort(current.begin(), current.end(), SocketAddressCompact_less());
