@@ -45,13 +45,14 @@
 #include "data/hash_torrent.h"
 #include "protocol/handshake_manager.h"
 #include "data/hash_queue.h"
-#include "net/throttle_manager.h"
 #include "net/listen.h"
 
 #include "torrent/chunk_manager.h"
 #include "torrent/connection_manager.h"
+#include "torrent/dht_manager.h"
 #include "torrent/data/file_manager.h"
 #include "torrent/peer/client_list.h"
+#include "torrent/throttle.h"
 
 #include "manager.h"
 #include "resource_manager.h"
@@ -70,11 +71,12 @@ Manager::Manager() :
   m_chunkManager(new ChunkManager),
   m_clientList(new ClientList),
   m_connectionManager(new ConnectionManager),
+  m_dhtManager(new DhtManager),
 
   m_poll(NULL),
 
-  m_uploadThrottle(new ThrottleManager),
-  m_downloadThrottle(new ThrottleManager),
+  m_uploadThrottle(Throttle::create_throttle()),
+  m_downloadThrottle(Throttle::create_throttle()),
 
   m_ticks(0) {
 
@@ -99,13 +101,14 @@ Manager::~Manager() {
   delete m_hashQueue;
 
   delete m_resourceManager;
+  delete m_dhtManager;
   delete m_connectionManager;
   delete m_chunkManager;
 
   delete m_clientList;
 
-  delete m_uploadThrottle;
-  delete m_downloadThrottle;
+  Throttle::destroy_throttle(m_uploadThrottle);
+  Throttle::destroy_throttle(m_downloadThrottle);
 }
 
 void
