@@ -45,6 +45,9 @@
 #include "download/chunk_statistics.h"
 #include "download/download_info.h"
 #include "download/download_main.h"
+#include "torrent/dht_manager.h"
+#include "torrent/peer/connection_list.h"
+#include "torrent/peer/peer_info.h"
 
 #include "extensions.h"
 #include "peer_connection_leech.h"
@@ -122,13 +125,14 @@ PeerConnectionLeech::receive_keepalive() {
   // again.
 
   // TODO: Do we need to remove from download throttle here?
+  //
+  // Do we also need to remove from download throttle? Check how it
+  // worked again.
 
-  if (!download_queue()->empty() && m_downStall++ != 0) {
-    if (m_downStall > 6)
-      download_queue()->cancel();
-    else
-      download_queue()->stall();
-  }
+  if (!download_queue()->canceled_empty() && m_downStall >= 6)
+    download_queue()->cancel();
+  else if (!download_queue()->queued_empty() && m_downStall++ != 0)
+    download_queue()->stall();
 
   return true;
 }
@@ -275,6 +279,13 @@ PeerConnectionLeech::read_message() {
     read_cancel_piece(m_down->read_request());
     return true;
 
+  case ProtocolBase::PORT:
+    if (!m_down->can_read_port_body())
+      break;
+
+    manager->dht_manager()->add_node(m_peerInfo->socket_address(), m_down->buffer()->read_16());
+    return true;
+
   case ProtocolBase::EXTENSION_PROTOCOL:
     if (!m_down->can_read_extension_body())
       break;
@@ -487,13 +498,15 @@ PeerConnectionLeech::fill_write_buffer() {
     m_peerChunks.cancel_queue()->pop_front();
   }
 
-  if (m_sendPEXMask && m_up->can_write_extension())
-    send_pex_message();
+  if (m_sendPEXMask && m_up->can_write_extension() &&
+      send_pex_message()) {
+    // Don't do anything else if send_pex_message() succeeded.
 
-  else if (!m_upChoke.choked() &&
-           !m_peerChunks.upload_queue()->empty() &&
-           m_up->can_write_piece())
+  } else if (!m_upChoke.choked() &&
+             !m_peerChunks.upload_queue()->empty() &&
+             m_up->can_write_piece()) {
     write_prepare_piece();
+  }
 
   if (is_encrypted())
     m_encryption.encrypt(old_end, m_up->buffer()->end() - old_end);
