@@ -225,7 +225,7 @@ FileList::split(iterator position, split_type* first, split_type* last) {
     newFile->set_offset(offset);
     newFile->set_size_bytes(first->first);
     newFile->set_range(m_chunkSize);
-    *newFile->path() = first->second;
+    *newFile->mutable_path() = first->second;
 
     offset += first->first;
     *itr = newFile;
@@ -247,7 +247,7 @@ FileList::merge(iterator first, iterator last, const Path& path) {
 
   // Set the path before deleting any iterators in case it refers to
   // one of the objects getting deleted.
-  *newFile->path() = path;
+  *newFile->mutable_path() = path;
 
   if (first == last) {
     if (first == end())
@@ -274,12 +274,12 @@ FileList::merge(iterator first, iterator last, const Path& path) {
   if (first == begin())
     newFile->set_match_depth_prev(0);
   else
-    set_match_depth(*(first - 1), newFile);
+    File::set_match_depth(*(first - 1), newFile);
 
   if (first + 1 == end())
     newFile->set_match_depth_next(0);
   else
-    set_match_depth(newFile, *(first + 1));
+    File::set_match_depth(newFile, *(first + 1));
 
   return first;
 }
@@ -293,20 +293,58 @@ FileList::update_paths(iterator first, iterator last) {
     return;
 
   if (first != begin())
-    set_match_depth(*(first - 1), *first);
+    File::set_match_depth(*(first - 1), *first);
 
   while (first != last && ++first != end())
-    set_match_depth(*(first - 1), *first);
+    File::set_match_depth(*(first - 1), *first);
 
   verify_file_list(this);
 }
 
-void
-FileList::set_file_completed_chunks(iterator itr, uint32_t v) {
-  if (is_open())
-    return;
+bool
+FileList::make_root_path() {
+  if (!is_open())
+    return false;
 
-  (*itr)->set_completed(v);
+  return ::mkdir(m_rootDir.c_str(), 0777) == 0 || errno == EEXIST;
+}
+
+bool
+FileList::make_all_paths() {
+  if (!is_open())
+    return false;
+
+  Path dummyPath;
+  const Path* lastPath = &dummyPath;
+
+  for (iterator itr = begin(), last = end(); itr != last; ++itr) {
+    File* entry = *itr;
+
+    // No need to create directories if the entry has already been
+    // opened.
+    if (entry->is_open())
+      continue;
+
+    if (entry->path()->empty())
+      throw storage_error("Found an empty filename.");
+
+    Path::const_iterator lastPathItr   = lastPath->begin();
+    Path::const_iterator firstMismatch = entry->path()->begin();
+
+    // Couldn't find a suitable stl algo, need to write my own.
+    while (firstMismatch != entry->path()->end() && lastPathItr != lastPath->end() && *firstMismatch == *lastPathItr) {
+      lastPathItr++;
+      firstMismatch++;
+    }
+
+    rak::error_number::clear_global();
+
+    make_directory(entry->path()->begin(), entry->path()->end(), firstMismatch);
+    
+    lastPath = entry->path();
+  }
+
+  return true;
 }
 
 // Initialize FileList and add a dummy file that may be split into
@@ -341,7 +379,7 @@ struct file_list_cstr_less {
 };
 
 void
-FileList::open() {
+FileList::open(int flags) {
   typedef std::set<const char*, file_list_cstr_less> path_set;
 
   if (m_rootDir.empty())
@@ -351,20 +389,23 @@ FileList::open() {
 
   Path lastPath;
   path_set pathSet;
-  iterator itr = begin();
 
   try {
-    if (::mkdir(m_rootDir.c_str(), 0777) != 0 && errno != EEXIST)
+    if (!(flags & open_no_create) && !make_root_path())
       throw storage_error("Could not create directory '" + m_rootDir + "': " + std::strerror(errno));
   
-    while (itr != end()) {
-      File* entry = *itr++;
+    for (iterator itr = begin(), last = end(); itr != last; ++itr) {
+      File* entry = *itr;
 
+      // We no longer consider it an error to open a previously opened
+      // FileList as we now use the same function to create
+      // non-existent files.
+      //
+      // Since m_isOpen is set, we know root dir wasn't changed, thus
+      // we can keep the previously opened file.
       if (entry->is_open())
-        throw internal_error("FileList::open(...) found an already opened file.");
+        continue;
       
-//       manager->file_manager()->insert(entry);
-
       // Update the path during open so that any changes to root dir
       // and file paths are properly handled.
       if (entry->path()->back().empty())
@@ -381,17 +422,35 @@ FileList::open() {
       if (entry->path()->empty())
         throw storage_error("Found an empty filename.");
 
-      if (!open_file(&*entry, lastPath))
-        throw storage_error("Could not open file \"" + m_rootDir + entry->path()->as_string() + "\": " + rak::error_number::current().c_str());
-      
+      // Handle directory creation outside of open_file, so we can do
+      // it here if necessary.
+
+      entry->set_flags_protected(File::flag_active);
+
+      if (!open_file(&*entry, lastPath, flags)) {
+        // This needs to check if the error was due to open_no_create
+        // being set or not.
+        if (!(flags & open_no_create))
+          // Also check if open_require_all_open is set.
+          throw storage_error("Could not open file \"" + m_rootDir + entry->path()->as_string() + "\": " + rak::error_number::current().c_str());
+
+        // Don't set the lastPath as we haven't created the directory.
+        continue;
+      }
+
       lastPath = *entry->path();
     }
 
-  } catch (storage_error& e) {
-    for (iterator cleanupItr = begin(); cleanupItr != itr; ++cleanupItr)
-      manager->file_manager()->close(*cleanupItr);
+  } catch (local_error& e) {
+    for (iterator itr = begin(), last = end(); itr != last; ++itr) {
+      (*itr)->unset_flags_protected(File::flag_active);
+      manager->file_manager()->close(*itr);
+    }
 
-    throw e;
+    // Set to false here in case we tried to open the FileList for the
+    // second time.
+    m_isOpen = false;
+    throw;
   }
 
   m_isOpen = true;
@@ -403,29 +462,14 @@ FileList::close() {
     return;
 
   for (iterator itr = begin(), last = end(); itr != last; ++itr) {
+    (*itr)->unset_flags_protected(File::flag_active);
     manager->file_manager()->close(*itr);
-    
-    // Keep the progress so the user can see it even though he closes
-    // the torrent.
-//     (*itr)->set_completed(0);
   }
 
   m_isOpen = false;
   m_indirectLinks.clear();
 }
 
-bool
-FileList::resize_all() {
-  bool success = true;
-
-  for (iterator itr = begin(); itr != end(); itr++)
-    if (!(*itr)->frozen_path().empty() &&
-        !(*itr)->resize_file())
-      success = false;
-
-  return success;
-}
-
 void
 FileList::make_directory(Path::const_iterator pathBegin, Path::const_iterator pathEnd, Path::const_iterator startItr) {
   std::string path = m_rootDir;
@@ -454,25 +498,27 @@ FileList::make_directory(Path::const_iterator pathBegin, Path::const_iterator pa
 }
 
 bool
-FileList::open_file(File* node, const Path& lastPath) {
-  const Path* path = node->path();
-
-  Path::const_iterator lastItr = lastPath.begin();
-  Path::const_iterator firstMismatch = path->begin();
-
-  // Couldn't find a suitable stl algo, need to write my own.
-  while (firstMismatch != path->end() && lastItr != lastPath.end() && *firstMismatch == *lastItr) {
-    lastItr++;
-    firstMismatch++;
-  }
-
+FileList::open_file(File* node, const Path& lastPath, int flags) {
   rak::error_number::clear_global();
 
-  make_directory(path->begin(), path->end(), firstMismatch);
+  if (!(flags & open_no_create)) {
+    const Path* path = node->path();
+
+    Path::const_iterator lastItr = lastPath.begin();
+    Path::const_iterator firstMismatch = path->begin();
+
+    // Couldn't find a suitable stl algo, need to write my own.
+    while (firstMismatch != path->end() && lastItr != lastPath.end() && *firstMismatch == *lastItr) {
+      lastItr++;
+      firstMismatch++;
+    }
+
+    make_directory(path->begin(), path->end(), firstMismatch);
+  }
 
   // Some torrents indicate an empty directory by having a path with
   // an empty last element. This entry must be zero length.
-  if (path->back().empty())
+  if (node->path()->back().empty())
     return node->size_bytes() == 0;
 
   rak::file_stat fileStat;
@@ -486,8 +532,8 @@ FileList::open_file(File* node, const Path& lastPath) {
   }
 
   return
-    node->prepare(MemoryChunk::prot_read | MemoryChunk::prot_write, SocketFile::o_create) ||
-    node->prepare(MemoryChunk::prot_read, SocketFile::o_create);
+    node->prepare(MemoryChunk::prot_read | MemoryChunk::prot_write, (flags & open_no_create ? 0 : SocketFile::o_create)) ||
+    node->prepare(MemoryChunk::prot_read, 0);
 }
 
 MemoryChunk
@@ -574,7 +620,7 @@ FileList::inc_completed(iterator firstItr, uint32_t index) {
 
   std::for_each(firstItr,
                 lastItr == end() ? end() : (lastItr + 1),
-                std::mem_fun(&File::inc_completed));
+                std::mem_fun(&File::inc_completed_protected));
 
   return lastItr;
 }
@@ -586,13 +632,13 @@ FileList::update_completed() {
 
   if (m_bitfield.is_all_set()) {
     for (iterator itr = begin(), last = end(); itr != last; ++itr)
-      (*itr)->set_completed((*itr)->size_chunks());
+      (*itr)->set_completed_protected((*itr)->size_chunks());
 
   } else {
     // Clear any old progress data from the entries as we don't clear
     // this on close, etc.
     for (iterator itr = begin(), last = end(); itr != last; ++itr)
-      (*itr)->set_completed(0);
+      (*itr)->set_completed_protected(0);
 
     if (m_bitfield.is_all_unset())
       return;
@@ -605,21 +651,4 @@ FileList::update_completed() {
   }
 }
 
-void
-FileList::set_match_depth(File* left, File* right) {
-  uint32_t level = 0;
-
-  Path::const_iterator itrLeft = left->path()->begin();
-  Path::const_iterator itrRight = right->path()->begin();
-
-  while (itrLeft != left->path()->end() && itrRight != right->path()->end() && *itrLeft == *itrRight) {
-    itrLeft++;
-    itrRight++;
-    level++;
-  }
-
-  left->set_match_depth_next(level);
-  right->set_match_depth_prev(level);
-}
-
 }
