@@ -54,6 +54,12 @@
 
 namespace torrent {
 
+TransferList::TransferList() :
+  m_slotCanceled(slot_canceled_type(slot_canceled_op(NULL), NULL)),
+  m_slotCompleted(slot_completed_type(slot_completed_op(NULL), NULL)),
+  m_slotQueued(slot_queued_type(slot_queued_op(NULL), NULL)),
+  m_slotCorrupt(slot_corrupt_type(slot_corrupt_op(NULL), NULL)) { }
+
 TransferList::iterator
 TransferList::find(uint32_t index) {
   return std::find_if(begin(), end(), rak::equal(index, std::mem_fun(&BlockList::index)));
